// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: https://svn/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Algorithms/ITK/include/mapITKSVMetricControlInterface.h $
*/




#ifndef __ITK_SV_METRIC_CONTROL_INTERFACE_H
#define __ITK_SV_METRIC_CONTROL_INTERFACE_H

#include "mapITKMetricControlInterface.h"

#include "itkSingleValuedCostFunction.h"


namespace map
{
	namespace algorithm
	{
		namespace itk
		{

			/*! @class ITKSVMetricControlInterface
			@brief Interface for controller classes that form a virtual wrapper around itk::SingleValuedCostFunctions
			@ingroup ITK
			*/
			class MAPAlgorithmsITK_EXPORT ITKSVMetricControlInterface : public ITKMetricControlInterface
			{
			public:
				typedef ITKSVMetricControlInterface Self;
				typedef ITKMetricControlInterface Superclass;
				typedef ::itk::SmartPointer<Self>         Pointer;
				typedef ::itk::SmartPointer<const Self>   ConstPointer;

				itkTypeMacro(ITKSVMetricControlInterface, ITKMetricControlInterface);

				typedef ::itk::SingleValuedCostFunction SVMetricBaseType;

				virtual SVMetricBaseType* getSVMetric() = 0;
				virtual const SVMetricBaseType* getSVMetric() const = 0;

			protected:
				ITKSVMetricControlInterface();
				virtual ~ITKSVMetricControlInterface();

			private:
				//No copy constructor allowed
				ITKSVMetricControlInterface(const Self& source);
				void operator=(const Self&);  //purposely not implemented
			};

		}
	}
}

#endif
